# -*- project-name: VASM -*-
package VASM::Catalog::XOrgConfig;

use strict;
use warnings;
#use base qw/VASM::Catalog::XorgConfig::Translation/;
use XML::Parser;
use XML::Writer;
use IO::File;
use Carp;

# Should hardware detection go in separate package? Yes. System interaction
# should be decoupled from data manipulation. This class is just for frobbing.

our $VERSION = '1.0';

sub new {
  my ($self, $handle) = @_; my $instance = {};

  bless $instance, $self;
  # If a file handle was given to parse:
  defined $handle and $instance->Parse($handle);

  return $self;
}

sub Store {
  # $key represents a particular aspect of X configuration, such as 'hsync'.
  # @list contains the values associated with it
  my ($self, $key, @list) = @_;
  
  croak 'Key and at least one list item must be given!'
    unless defined $key and @list;

  # It is now very simple to store the list in the catalog itself
  $self->{$key} = [ @list ];

  return;
}

sub Retrieve {
  my ($self, $key) = @_;

  # Check for key
  croak 'Key argument must be given!' unless defined $key;

  # If the key is defined, return a copy of its array
  defined $self->{$key} and return @{ $self->{$key} };
  return;
}

sub parseValueList {
  my ($self, $key, $tags) = @_; my @values;
 
  for my $index (grep { $_ % 2 } (0..$#{ $tags })) {
    next if $tags->[$index] eq '0'; # Skip text elements

    if ($tags->[$index] eq 'value') {
      # The textual content of the tag must have a length greater than 0
      croak 'Invalid or blank content in <value> tag'
        unless $tags->[$index + 1]->[1] eq '0'
          and length $tags->[$index + 1]->[2];
      # Add the value to @values in order
      push @values, $tags->[$index + 1]->[2];
    } else {
      croak 'Invalid tag in Xorg config catalog';
    }
  }

  # Intern the digested values
  $self->Store($key, @values);
  
  return;
}

sub Parse {
  my ($self, $handle) = @_;
  # If this fails, XML::Parser will emit an error message and end the program.
  # Unfortunately, this message is always English. :( There is at least an
  # error context, which can be universally understood by the programmer.
  my $parser = XML::Parser->new(Style => 'Tree', ErrorContext => 3);
  my $tree = $parser->parse($handle); my $tags = $tree->[1];

  # Main loop: iterate through tags under <catalog>
  for my $index (grep { $_ % 2 } (0..$#{ $tags })) {
    next if $tags->[$index] eq '0'; # Skip text elements

    # Accept any tag: pass its name as the first argument to parseValueList,
    # and the branch of the parsed XML structure corresponding to its values
    # as the second argument
    $self->parseValueList($tags->[$index], $tags->[$index + 1]->[0]);
  }
  
  return;
}

sub Write {
  my ($self, $handle)
}

1;

__END__

=head1 NAME

VASM::Catalog::XorgConfig

=head1 SYNOPSIS

=head1 DESCRIPTION

=head1 METHODS

=head1 FORMAT

=head1 AUTHORS

hanumizzle L<mailto:hanumizzle@gmail.com> wrote VASM::Catalog::XorgConfig.

=cut
